﻿using System;
using YEA.Infrastructure.Gateway;
using YEA.Monitor.LogService.States;
using YEA.Utilities.Service.Controller;

namespace YEA.Monitor.LogService
{
    public class ServiceStateController : IDisposable
    {
        protected RunningState runningState;
        protected DiagnosticState diagnosticState;
        protected RecoveryState recoveryState;

        protected IServiceState state;
        private bool disposed;


        internal ServiceStateController()
        {
            runningState = new RunningState();
            diagnosticState = new DiagnosticState();
            recoveryState = new RecoveryState();
            recoveryState.logger = runningState.logger;
            diagnosticState.logger = runningState.logger;
            ServiceStateBase.StatusChanged += ServiceStateBase_StatusChanged;
            state = runningState;
            disposed = false;
        }
        public ServiceStateController(IGatewayFactory gf)
            : this()
        {
            runningState.GatewayFactory = gf;
        }
        ~ServiceStateController()
        {
            Dispose(false);
        }

        void ServiceStateBase_StatusChanged(object sender, EventArgs<StateEventArgs> e)
        {
            switch (e.EventData.state)
            {
                case StateEvent.Errored: ChangeState(diagnosticState);
                    diagnosticState.SetException((Exception) e.EventData.stateDetails);
                    diagnosticState.Start();
                    break;
                case StateEvent.Diagnosed: ChangeState(recoveryState);
                    recoveryState.Diagnosis = (Diagnostic) e.EventData.stateDetails;
                    recoveryState.Start();
                    break;
                case StateEvent.Recovered: ChangeState(runningState);
                    runningState.Start();
                    break;
                case StateEvent.Stopped :
                    ChangeState(runningState);
                    //doesn't call the start automatically, waits for user to do so.
                    break;
               
            }
        }

        protected virtual void ChangeState(IServiceState newState)
        {
            state = newState;
        }

        public virtual void Start()
        {
            state.Start();
        }

        public virtual void Stop()
        {
            state.Stop();
        }

        public virtual void Pause()
        {
            state.Pause();
        }

        public virtual void Continue()
        {
            state.Continue();
        }


        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        protected void Dispose(bool disposing)
        {
            if (!disposed)
            {
                runningState.Dispose();
                diagnosticState.Dispose();
                recoveryState.Dispose();
            }
        }
    }
}
